﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ConsoleApplication2
{
    enum BroadcastStatus
    {
        StandBy = 0,
        OnAir = 1,
        End = 2
    }

    interface RadioBroadcaster//지켜볼 대상
    {
        bool ListenTo(Listener listener);
        bool QuitFrom(Listener listener);
        void DoBroadcasting();
        void StartBroadcast();
    }

    class KBRRadioBroadcaster : RadioBroadcaster//정보의 공급자
    {
        private List<Listener> listeners = new List<Listener>();
        public BroadcastStatus broadcastStatus = BroadcastStatus.StandBy;

        private List<string> scripts = new List<string>
        {
            "안녕하십니까. KBR방송국의 김병만 기자입니다.",
            "요즘 살림살이 초큼 어찌 나아지셨슴까?",
            "안 나아지셨다구요? 그럼 이 방송에서 취업 필살전략을 알려드립니다.",
            "요즘같이 취업이 어려운때 3개 국어가 가능하면 취업 직빵이겠죠.",
            "따라 해보시져, Handle いぱい 꺽어.(핸들 이빠이 꺽어).",
            "3개국어 정말 쉽져? 이것만 한다면 당신도 취업계의 슈퍼스타!"
        };

        private int currentScriptIndex = 0;

        public bool ListenTo(Listener listener)
        {
            try
            {
                listeners.Add(listener);
            }
            catch (Exception ex)
            {
                return false;
            }

            return true;
        }

        public bool QuitFrom(Listener listener)
        {
            if (listeners.Contains(listener))
            {
                listeners.Remove(listener);
                return true;
            }

            return false;
        }

        public void DoBroadcasting()
        {
            if (scripts.Count.Equals(currentScriptIndex))
            {
                Console.WriteLine("방송끗.");
                broadcastStatus = BroadcastStatus.End;
            }
            else
            {
                foreach (Listener listener in listeners)
                {
                    listener.Listening(scripts[currentScriptIndex]);
                }
                currentScriptIndex++;
            }
        }

        public void StartBroadcast()
        {
            broadcastStatus = BroadcastStatus.OnAir;
        }
    }

    class Listener//지켜보는 행위의 주체
    {
        private string name;

        public Listener(string name)
        {
            this.name = name;
            Console.WriteLine("나는야 {0}. 라디오를 가진 남자지.",this.name);
        }

        public void ListenToKBRRadio(RadioBroadcaster radioBroadcaster)
        {
            bool status = radioBroadcaster.ListenTo(this);

            if (status)
            {
                Console.WriteLine("{0} : 수신감도 조쿠나. ㅎㅎㅎㅎ", this.name);
            }
            else
            {
                Console.WriteLine("{0} : 아놕. 라디오도 못듣능구나", this.name);
            }
        }

        public void QuitFromKBRRadio(RadioBroadcaster radioBroadcaster)
        {
            bool status = radioBroadcaster.QuitFrom(this);

            if (status)
            {
                Console.WriteLine("{0} : 아놔. 완전 재미없엉", this.name);
            }
            else
            {
                Console.WriteLine("{0} : 아. 듣고 있던거 아니구낭", this.name);
            }
        }

        public void Listening(string script)
        {
            Console.WriteLine("{0} : {1}", this.name,script);
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            Listener listener1 = new Listener("마논개");
            Listener listener2 = new Listener("코턱용");
            Listener listener3 = new Listener("송순신");

            KBRRadioBroadcaster kbrRadioBroadcaster = new KBRRadioBroadcaster();
            listener1.ListenToKBRRadio(kbrRadioBroadcaster);//마논개 라디오 청취시작.
            int count = 0;
            do
            {
                if (count == 0)
                {
                    kbrRadioBroadcaster.StartBroadcast();
                }
                if (count == 2)
                {
                    listener2.ListenToKBRRadio(kbrRadioBroadcaster);//코턱용 라디오 청취시작
                }
                if (count == 3)
                {
                    listener1.QuitFromKBRRadio(kbrRadioBroadcaster);//마논개 라디오 끔
                    listener3.ListenToKBRRadio(kbrRadioBroadcaster);//송순신 라디오 청취시작
                }
                kbrRadioBroadcaster.DoBroadcasting();
                count++;
                Thread.Sleep(500);
            } while (kbrRadioBroadcaster.broadcastStatus != BroadcastStatus.End);
            
        }
    }
}
